;**********************************************************************
;                                                                     *
;    Filename:	    rx-003.asm                                        *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        el@jap.hu                                         *
;                   http://jap.hu/                                    *
;**********************************************************************
;    Notes:
;
;RF receiver prot. v2 development 1999/12/28
;receive digit is on RA0 (F84)
;supposed crystal freq is 4MHz
;bit #0 is 1.6kHz (delay 625 usec btw. slopes)
;bit #1 is 800 Hz
;**********************************************************************
;HISTORY
;
;000-19991229
;initial version, only displaytest
;
;001-19991229
;working version, no chksum or devid check
;only one errorcode: receive byte error (1)
;
;002-19991230
;more errorcodes: 3=chksum error, 4=devid mismatch
;f84 only
;
;003-without display, raw outputs
;code for both the c508 and f84
;**********************************************************************

;define F84
#define C508

;define RX1010
;polarity

#ifdef F84
	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
	;WDT=watchdog, PWRTE=power on timer, XT= crystal osc

freemem EQU 0x10
#define RXBIT PORTA, 0
ctlport	EQU PORTB

#endif
#ifdef C508
	list	p=12c508
	#include <p12C508.inc>

	__CONFIG   _CP_OFF & _WDT_OFF & _MCLRE_OFF & _IntRC_OSC

freemem EQU 0x07
#define RXBIT GPIO, 3
ctlport	EQU GPIO

#endif

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.


;***** VARIABLE DEFINITIONS

bt		EQU	freemem
cnt1		EQU	freemem+1
cnt2		EQU	freemem+2
cnt3		EQU	freemem+3
ptime		EQU	freemem+4
xptime		EQU	freemem+5

buffer		EQU	freemem+6
bdevid		EQU	buffer  ; 6 byte DEVID
bdata		EQU	buffer+6; 2 byte 'real' data
bsum		EQU	buffer+8; chksum

hdrcntmin	EQU	0x0c
hdrcntmax	EQU	8

min0		EQU	.50
min1		EQU	.115

;**********************************************************************
#ifdef F84
		ORG     0x000             ; processor reset vector
		nop
  		goto    main              ; go to beginning of program
		nop
		nop

		;ORG     0x004             ; interrupt vector location
		retfie
#endif

#ifdef C508
		ORG	0x000
		movwf	OSCCAL
		;C508 HARDWARE INIT
		clrf	GPIO ; clear all I/O pins
		clrw	;all except GP3 is output
		TRIS	GPIO ; I/O directions

		clrwdt
		movlw	1
		OPTION
		;-GPWU wakeup ENA
		;-GPPU pullup ENA on GP0,1,3
		;T0CS disable: enablings implicitly sets GP2 dir as input!
		;prescaler is 1:4, assigned to TMR0

		goto	main
#endif

table1		addwf PCL
devid		dt 0xc1, 0x06, 0x28, 0x64, 0xf1, 0x4e

main		
#ifdef F84
		;F84 HARDWARE INIT
		MOVLW 0 ;B00000000 ;SET LEVELS
		MOVWF PORTA
		MOVWF PORTB

		MOVLW 0 ; ;SET PORTB all out
		TRIS PORTB
		MOVLW 0xff
		TRIS PORTA ; PORTA all input

		clrwdt ; changing default presc. assignment
		movlw 0x81 ; prescaler 1:4 assigned to TMR0
		OPTION     ; T0CS selects internal CLK
#endif

warm		clrf ctlport ; clear all outputs after timeout

loop
		call wait4hdr
		andlw 0xff
		bnz l_err

l_ok
;recvbuf expanded for c508
recvbuf		movlw buffer
		movwf FSR
		movlw 9 ; number of bytes to receive
		movwf cnt2

recvb0		call recvbyte
		andlw 0xff ; test for return value
		bnz l_err
		movf bt, W ; received byte
		movwf INDF
		incf FSR, F
		decfsz cnt2, F
		goto recvb0
		; all bytes received OK
;recvbuf expanded for c508

		call chkbuf
		bnz l_err
		call chkdev
		andlw 0xff
		bnz l_err
		;received buffer is sane

		;put bit#3 into bit#5
		btfsc bdata, 3
		bsf bdata, 5

		;if a button is pressed, the corresponding
		;bit is 0. invert this here
		movf bdata, W
		xorlw 0xff
		movwf ctlport

		clrf xptime
		goto loop

l_err		;no valid header was received this cycle

		incfsz xptime, F ; clear outputs after 256
		goto loop        ; empty cycles

		goto warm        ; clear


wait_err2	retlw 2
wait4hdr	
		call measure_sil
		bz wait_err2 ; timeout

wait1		movlw hdrcntmin ; header bit counter
		movwf cnt1

wait0		call measure_sig
		bz wait_err2 ; timeout
		movlw min0
		subwf ptime, W
		bnc wait_err2 ; under min0
		movlw min1
		subwf ptime, W
		bc wait_err2 ; over min1, not bit0

		call measure_sil
		bz wait_err2 ; timeout
		movlw min0
		subwf ptime, W
		bnc wait1 ; under min0
		movlw min1
		subwf ptime, W
		bc wait1 ; over min1, not bit0

		; it was one possibly valid header bit
break0		decfsz cnt1, F
		goto wait0

		; the required minimum number of header bits
		; were received
		; now wait for a bit1 signal
		movlw hdrcntmax
		movwf cnt1
wait2		
		call measure_sig
		bz wait_err2 ; timeout, start over
		movlw min0
		subwf ptime, W
		bnc wait_err2 ; under min0, start over
		movlw min1
		subwf ptime, W
		bnc wait3 ; still bit0

		;found bit1
		retlw 0

wait3		call measure_sil
		bz wait_err2 ; timeout, start over
		movlw min0
		subwf ptime, W
		bnc wait1 ; under min0

		; it was still a possibly valid header bit0
break1		decfsz cnt1, F
		goto wait2

		; timeout in the second phase
		retlw 2

chkbuf		; check buffer sanity by chksum
		movlw buffer
		movwf FSR
		movlw 8 ; number of bytes - chksum
		movwf cnt2
		clrw

chk0		addwf INDF, W
		incf FSR, F
		decfsz cnt2, F
		goto chk0
		;chksum is now in W
		subwf bsum, W
		bnz chk_err
		retlw 0 ; result is in Z
chk_err		retlw 3 ; checksum error

chkdev		;check if the device id is the same
		movlw buffer
		movwf FSR
		clrf cnt2

chkd0		movf cnt2, W
		call table1
		subwf INDF, W
		bnz chkd_err
		incf FSR, F
		incf cnt2, F
		movlw 6 ; length of DEVID
		subwf cnt2, W
		bnz chkd0
		retlw 0 ; OK

chkd_err	retlw 4; devid error
		

recvbyte	;receive a byte into W
		movlw 8
		movwf cnt1; bit counter

recv0		call measure_sil ; the previous silence
		bz recv_err
		call measure_sig
		bz recv_err
		movlw min0
		subwf ptime, W
		bnc recv_err ; under min0

		movlw min1
		subwf ptime, W
		;received bit is in C now
		rlf bt, F

		decfsz cnt1, F
		goto recv0

		;received all 8 bits, now get an extra pulse
		call measure_sil
		bz recv_err
		call measure_sig
		bz recv_err
		retlw 0 ; OK

recv_err
recv_err1	retlw 1 ; receive error

#ifdef RX1010
measure_sil
#else
measure_sig	; measure signal time
#endif
		movlw 1
		movwf TMR0
ms0		movf TMR0, W
		btfsc STATUS, Z
		goto timeout
		btfsc RXBIT ; test receive bit state
		goto ms0    ; loop while SET
		movwf ptime
		retlw 0
#ifdef RX1010
measure_sig
#else
measure_sil	; measure silence time
#endif
		movlw 1
		movwf TMR0
ms1		movf TMR0, W
		btfsc STATUS, Z
		goto timeout
		btfss RXBIT ; test receive bit state
		goto ms1    ; loop while LOW
		movwf ptime
		retlw 0

timeout		;timeout returns W=2
		retlw 2


		end

